@@ -19,7 +19,6 @@
 void be2iscsi_create_conn(struct iscsi_conn *conn)
 {
 	struct iscsi_session *session = conn->session;
-	conn_rec_t *conn_rec = &session->nrec.conn[conn->id];
 
 	if (conn->max_recv_dlength > 65536)
 		conn->max_recv_dlength = 65536;
@@ -33,10 +32,6 @@ void be2iscsi_create_conn(struct iscsi_conn *conn)
 	if (conn->max_xmit_dlength > 65536)
 		conn->max_xmit_dlength = 65536;
 
-	if (!conn_rec->iscsi.MaxXmitDataSegmentLength ||
-	    conn_rec->iscsi.MaxXmitDataSegmentLength > 65536)
-		conn_rec->iscsi.MaxXmitDataSegmentLength = 65536;
-
 	session->erl = 0;
 	session->initial_r2t_en = 1;
 }
