@@ -23,7 +23,7 @@
  */
 #include <stdlib.h>
 #include <errno.h>
-#include <sys/poll.h>
+#include <poll.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/signalfd.h>
