@@ -301,7 +301,7 @@ static void iscsid_shutdown(void)
 	while ((pid = waitpid(0, NULL, 0) > 0))
 		log_debug(7, "cleaned up pid %d", pid);
 
-	log_warning("iscsid shutting down.");
+	log_info("iscsid shutting down.");
 	if (daemonize && log_pid >= 0) {
 		log_debug(1, "daemon stopping");
 		log_close(log_pid);
@@ -310,7 +310,12 @@ static void iscsid_shutdown(void)
 
 static void catch_signal(int signo)
 {
-	log_debug(1, "pid %d caught signal %d", getpid(), signo);
+	/*
+	 * Do not try to call log_debug() if there is a PIPE error
+	 * because we can get caught in a PIPE error loop.
+	 */
+	if (signo != SIGPIPE)
+		log_debug(1, "pid %d caught signal %d", getpid(), signo);
 
 	/* In foreground mode, treat SIGINT like SIGTERM */
 	if (!daemonize && signo == SIGINT)
