@@ -30,7 +30,6 @@
 #include <time.h>
 #include <sys/mman.h>
 #include <sys/utsname.h>
-#include <sys/signal.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 
@@ -348,6 +347,7 @@ int main(int argc, char *argv[])
 	struct boot_context *context, boot_context;
 	struct sigaction sa_old;
 	struct sigaction sa_new;
+	struct user_param *param;
 	int control_fd, mgmt_ipc_fd, err;
 	pid_t pid;
 
@@ -542,6 +542,10 @@ int main(int argc, char *argv[])
 	mgmt_ipc_close(mgmt_ipc_fd);
 	free_initiator();
 	sysfs_cleanup();
+	list_for_each_entry(param, &user_params, list) {
+		list_del(&param->list);
+		idbm_free_user_param(param);
+	}
 
 	log_debug(1, "iscsi child done");
 	return 0;
